
//#include <MySensors.h>
#include <nrf.h>
uint8_t myBuffer[20];  //required buffer for transmitting packet
uint8_t my_default_network_address[] = {0xE7, 0xE7, 0xE7, 0xE7, 0xE7};

/*
 * nRF5 output modes
 */
// Standard 0, Standard 1
#ifndef OUTPUT_S0S1
#define OUTPUT_S0S1  (0x10)
#endif

// High Drive 0, Standard 1
#ifndef OUTPUT_H0S1
#define OUTPUT_H0S1 (0x11)
#endif

// Standard 0, High Drive 1
#ifndef OUTPUT_S0H1
#define OUTPUT_S0H1 (0x12)
#endif

// High Drive both
#ifndef OUTPUT_H0H1
#define OUTPUT_H0H1 (0x13)
#endif

// Disconnected 0, Standard 1
#ifndef OUTPUT_D0S1
#define OUTPUT_D0S1 (0x14)
#endif

// Disconnected 0, High Drive 1
#ifndef OUTPUT_D0H1
#define OUTPUT_D0H1 (0x15)
#endif

// Standard 0, Disconnected 1
#ifndef OUTPUT_S0D1
#define OUTPUT_S0D1 (0x16)
#endif

// High Drive 0, Disconnected 1
#ifndef OUTPUT_H0D1
#define OUTPUT_H0D1 (0x17)
#endif


#ifdef __cplusplus
extern "C" {
#endif
void myNrf5_pinMode(uint32_t ulPin, uint32_t ulMode)
{
  if (ulPin >= PINS_COUNT) {
    return;
  }

  ulPin = g_ADigitalPinMap[ulPin];

  // Set pin mode according to chapter '22.6.3 I/O Pin Configuration'
  switch (ulMode) {
  case INPUT:
    // Set pin to input mode
    NRF_GPIO->PIN_CNF[ulPin] =
        ((uint32_t)GPIO_PIN_CNF_DIR_Input << GPIO_PIN_CNF_DIR_Pos) |
        ((uint32_t)GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos) |
        ((uint32_t)GPIO_PIN_CNF_PULL_Disabled << GPIO_PIN_CNF_PULL_Pos) |
        ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos) |
        ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
    break;

  case INPUT_PULLUP:
    // Set pin to input mode with pull-up resistor enabled
    NRF_GPIO->PIN_CNF[ulPin] =
        ((uint32_t)GPIO_PIN_CNF_DIR_Input << GPIO_PIN_CNF_DIR_Pos) |
        ((uint32_t)GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos) |
        ((uint32_t)GPIO_PIN_CNF_PULL_Pullup << GPIO_PIN_CNF_PULL_Pos) |
        ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos) |
        ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
    break;

  case INPUT_PULLDOWN:
    // Set pin to input mode with pull-down resistor enabled
    NRF_GPIO->PIN_CNF[ulPin] =
        ((uint32_t)GPIO_PIN_CNF_DIR_Input << GPIO_PIN_CNF_DIR_Pos) |
        ((uint32_t)GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos) |
        ((uint32_t)GPIO_PIN_CNF_PULL_Pulldown << GPIO_PIN_CNF_PULL_Pos) |
        ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos) |
        ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
    break;

  case OUTPUT:
    // Set pin to output mode
    NRF_GPIO->PIN_CNF[ulPin] =
        ((uint32_t)GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos) |
        ((uint32_t)GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos) |
        ((uint32_t)GPIO_PIN_CNF_PULL_Disabled << GPIO_PIN_CNF_PULL_Pos) |
        ((uint32_t)GPIO_PIN_CNF_DRIVE_S0S1 << GPIO_PIN_CNF_DRIVE_Pos) |
        ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
    break;

  default:
    // calculate nRF specific output modes
    if ((ulMode >= OUTPUT_S0S1) && (ulMode <= OUTPUT_H0D1)) {
      // Set pin to given output mode
      NRF_GPIO->PIN_CNF[ulPin] =
          ((uint32_t)GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos) |
          ((uint32_t)GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos) |
          ((uint32_t)GPIO_PIN_CNF_PULL_Disabled << GPIO_PIN_CNF_PULL_Pos) |
          ((uint32_t)(ulMode - OUTPUT_S0S1) << GPIO_PIN_CNF_DRIVE_Pos) |
          ((uint32_t)GPIO_PIN_CNF_SENSE_Disabled << GPIO_PIN_CNF_SENSE_Pos);
    }
    break;
  }
}

#ifdef __cplusplus
}
#endif


bool mySetNetworkAddress(uint8_t* address, uint8_t len)
{
    if (len < 3 || len > 5)
  return false;

    // First byte is the prefix, remainder are base
    NRF_RADIO->PREFIX0    = ((address[0] << RADIO_PREFIX0_AP0_Pos) & RADIO_PREFIX0_AP0_Msk);
    uint32_t base;
    memcpy(&base, address+1, len-1);
    NRF_RADIO->BASE0 = base;

    NRF_RADIO->PCNF1 =  (
  (((sizeof(myBuffer)) << RADIO_PCNF1_MAXLEN_Pos)  & RADIO_PCNF1_MAXLEN_Msk)  // maximum length of payload
  | (((0UL)        << RADIO_PCNF1_STATLEN_Pos) & RADIO_PCNF1_STATLEN_Msk) // expand the payload with 0 bytes
  | (((len-1)      << RADIO_PCNF1_BALEN_Pos)   & RADIO_PCNF1_BALEN_Msk)); // base address length in number of bytes.

    return true;
}

void myInit() {
      // Enable the High Frequency clock to the system as a whole
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART = 1;
    /* Wait for the external oscillator to start up */
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0)
  ;
    
    // Disable and reset the radio
    NRF_RADIO->POWER = RADIO_POWER_POWER_Disabled;
    NRF_RADIO->POWER = RADIO_POWER_POWER_Enabled;
    NRF_RADIO->EVENTS_DISABLED = 0;
    NRF_RADIO->TASKS_DISABLE   = 1;
    // Wait until we are in DISABLE state
    while (NRF_RADIO->EVENTS_DISABLED == 0) {}

    // Physical on-air address is set in PREFIX0 + BASE0 by setNetworkAddress
    NRF_RADIO->TXADDRESS    = 0x00; // Use logical address 0 (PREFIX0 + BASE0)
    NRF_RADIO->RXADDRESSES  = 0x01; // Enable reception on logical address 0 (PREFIX0 + BASE0)

    // Configure the CRC
    NRF_RADIO->CRCCNF = (RADIO_CRCCNF_LEN_Two << RADIO_CRCCNF_LEN_Pos); // Number of checksum bits
    NRF_RADIO->CRCINIT = 0xFFFFUL;      // Initial value      
    NRF_RADIO->CRCPOLY = 0x11021UL;     // CRC poly: x^16+x^12^x^5+1

    // These shorts will make the radio transition from Ready to Start to Disable automatically
    // for both TX and RX, which makes for much shorter on-air times
    //NRF_RADIO->SHORTS = (RADIO_SHORTS_READY_START_Enabled << RADIO_SHORTS_READY_START_Pos)
      //          | (RADIO_SHORTS_END_DISABLE_Enabled << RADIO_SHORTS_END_DISABLE_Pos);

    NRF_RADIO->PCNF0 =   (8 << RADIO_PCNF0_LFLEN_Pos) // Payload size length in bits
                 | (1 << RADIO_PCNF0_S0LEN_Pos) // S0 is 1 octet
                 | (8 << RADIO_PCNF0_S1LEN_Pos); // S1 is 1 octet
}

void setupMyBuffer() {
  myBuffer[0]=0;
  myBuffer[1]=9;  //length of this buffer
  myBuffer[2]=0;
  myBuffer[3]=255;
  myBuffer[4]=255;
  myBuffer[5]=0;
  myBuffer[6]=0;
  myBuffer[7]=72;    //=H
  myBuffer[8]=0;     //limit string to a single character  
  myBuffer[9]=108;   //=l
  myBuffer[10]=108;  //=l
  myBuffer[11]=111;  //=o
  myBuffer[12]=32;   //= 
  myBuffer[13]=87;   //=W
  myBuffer[14]=111;  //=o
  myBuffer[15]=114;  //=r
  myBuffer[16]=108;  //=l
  myBuffer[17]=100;  //=d
  myBuffer[18]=33;   //=!
  myBuffer[19]=0;
}

void setup() 
{ 
  NRF_POWER->DCDCEN=1;  //enable the DCDC voltage regulator as the default.
  setupMyBuffer();

  pinMode(PIN_BUTTON1, INPUT);
  if (digitalRead(PIN_BUTTON1)==1)  {
    myNrf5_pinMode(LED_BUILTIN,OUTPUT_D0H1);
    digitalWrite(LED_BUILTIN,HIGH);
    myBuffer[7]='U';  //'U' for "Up" button has been pressed.
  }
  else {
    myBuffer[7]='D';  //'D' for "Down" button has been pressed.
  }
  
  
  Serial.begin(250000);
  Serial.println();
  Serial.println("Starting the transmitter....");
  
  myInit();
  NRF_RADIO->PACKETPTR = (uint32_t)myBuffer;
  Serial.println("Finished myInit()");


  /*
  NRF_POWER->DCDCEN=1;  //enable the DCDC voltage regulator as the default.

  //guarantee RESET pin is working
  if (((NRF_UICR-> PSELRESET[0])==0xFFFFFFFF) && ((NRF_UICR-> PSELRESET[1])==0xFFFFFFFF)) { //if the two RESET registers are erased
    NRF_NVMC->CONFIG=1;  // Write enable the UICR
    NRF_UICR-> PSELRESET[0]=21;  //designate pin P0.21 as the RESET pin
    NRF_UICR-> PSELRESET[1]=21;  //designate pin P0.21 as the RESET pin
    NRF_NVMC->CONFIG=0;  // Put the UICR back into read-only mode.
  }
  */


  
  NRF_RADIO->TASKS_DISABLE=1;  //turn-off the radio to establish known state.
  while (NRF_RADIO->STATE!=0) {}  //busy-wait until radio is disabled
  

  NRF_RADIO->FREQUENCY=123;
  NRF_RADIO->MODE=1;  //set 2Mbps datarate. 
   
  //NRF_RADIO->SHORTS = B100001;  //Implement shortcuts: READY_START and END_START
  
  NRF_RADIO->TASKS_TXEN=1;  //wake-up the radio transmitter and move it into state TXIDLE.
  while ((NRF_RADIO->STATE)!=10) {}  //busy-wait until radio has started TXIDLE
  
  //The shortcuts will take-over the moment the state TXIDLE becomes activated.

  //NRF_RADIO->TASKS_DISABLE=1;  //turn-off the radio to establish known state.
  //while (NRF_RADIO->STATE!=0) {}  //busy-wait until radio is disabled

  mySetNetworkAddress(my_default_network_address, sizeof(my_default_network_address));
  Serial.println("Exiting steup...");
  Serial.println();
  Serial.flush();
}

uint32_t loopCounter=0;
void loop() {
    //assume radio is in TXIDLE state.
    //Serial.println("Start of loop...");
    //Serial.flush();
    //NRF_RADIO->TASKS_TXEN=1;  //wake-up the radio
    //while ((NRF_RADIO->STATE)!=10) {}  //busy-wait until radio has started TXIDLE
  
    NRF_RADIO->TASKS_START=1;  //Move from TXIDLE state to TX state.  This sends a NULL packet.
    while ((NRF_RADIO->STATE)!=11) {}  //busy-wait until radio is in TX state
    while ((NRF_RADIO->STATE)==11) {}  //busy-wait until radio is back to TXIDLE state
    //NRF_RADIO->TASKS_DISABLE=1;  //turn-off the radio to establish known state.
    //while (NRF_RADIO->STATE!=0) {}  //busy-wait until radio is disabled

    //Serial.print(loopCounter++);
    //Serial.println(", Sent a packet.");
    //Serial.flush();
   // delay(1000);
    //Assertion: radio is now back to TXIDLE state
}

